package polyglot.ext.ibex.lr;

import java.util.*;
import polyglot.main.Report;

// Each node in the graph is a pair <state,top>,
// where state represents the current parser state
// and top represents the top of the parser stack in that state.
//
// Actually, state is the top of stack and top is the state below
// the top.  We simulate the top two elements of the stack.
class SRNode extends NFAState implements Node {
    State state;
    State top;

    SRNode(State state, State top, int index) {
        super(index);

        this.state = state;
        this.top = top;

        // simplify:
        this.top = null;
    }

    public boolean equals(Object o) {
        return o instanceof SRNode
            && ((SRNode) o).index == index
            && ((SRNode) o).state.equals(state)
            && ( ((SRNode) o).top != null
                  ? ((SRNode) o).top.equals(top)
                  : top == null );
    }

    public int hashCode() {
        return state.hashCode() + index +
            (top != null ? top.hashCode() : 0);
    }

    public String toString() {
        if (top == null) {
            return "" + state.index();
        }
        else {
            return "<" + state.index() + "," + top.index() + ">";
        }
    }

    public int index() {
        return index;
    }

    public State state() {
        return state;
    }
}
