package polyglot.ext.ibex.types;

import polyglot.types.*;
import polyglot.ext.jl.types.*;
import polyglot.util.*;
import polyglot.frontend.Source;
import java.util.*;

public class MutableParserType_c extends ParsedClassType_c implements MutableParserType {
    List nonterminals;
    List terminals;
    Nonterminal startSymbol;

    protected MutableParserType_c() {
        super();
    }

    public MutableParserType_c(TypeSystem ts, LazyClassInitializer init,
                             Source fromSource) {
        super(ts, init, fromSource);
        nonterminals = new TypedList(new LinkedList(), Nonterminal.class, false);
        terminals = new TypedList(new LinkedList(), Terminal.class, false);
    }

    public void addNonterminal(Nonterminal nonterminal) {
        nonterminals().add(nonterminal);
    }

    public void addTerminal(Terminal terminal) {
        terminals().add(terminal);
    }

    public void setStartSymbol(Nonterminal start) {
        startSymbol = start;
    }

    public List nonterminals() {
        return nonterminals;
    }
        
    public List terminals() {
        return terminals;
    }
        
    public Nonterminal startSymbol() {
        return startSymbol;
    }

    public Terminal terminalNamed(String name) {
        for (Iterator i = terminals.iterator(); i.hasNext(); ) {
            Terminal t = (Terminal) i.next();
            if (t.name().equals(name)) {
                return t;
            }
        }
        return null;
    }

    public Nonterminal nonterminalNamed(String name) {
        for (Iterator i = nonterminals.iterator(); i.hasNext(); ) {
            Nonterminal t = (Nonterminal) i.next();
            if (t.name().equals(name)) {
                return t;
            }
        }
        return null;
    }

    public boolean isParser() {
        return ( ! nonterminals.isEmpty() && ! terminals.isEmpty() )
            || ( superType() instanceof ParserType &&
                 ((ParserType) superType()).isParser() );
    }

    public List allTerminals() {
        List l = new ArrayList();
        if (superType() instanceof ParserType) {
            l.addAll(((ParserType) superType()).allTerminals());
        }
        l.addAll(terminals);
        return l;
    }

    public List allNonterminals() {
        List l = new ArrayList();
        if (superType() instanceof ParserType) {
            l.addAll(((ParserType) superType()).allNonterminals());
        }
        l.addAll(nonterminals);
        return l;
    }
}
