/*******************************************************************************
 * The old PolyJAssign_c was divided into 3 classes (similar to what JL did):
 * PolyJLocalAssign_c, PolyJFieldAssign_c, PolyJArrayAccessAssign_c; only
 * difference is the tranlate function;
 ******************************************************************************/

package polyglot.ext.polyj.ast;

import polyglot.ext.polyj.extension.*;
import polyglot.ext.polyj.types.*;
import polyglot.ext.polyj.util.*;
import polyglot.ext.polyj.visit.*;

import polyglot.ext.jl.ast.*;

import polyglot.ast.*;
import polyglot.util.*;
import polyglot.types.*;
import polyglot.types.Package;
import polyglot.visit.*;

import java.util.*;

public class PolyJFieldAssign_c extends FieldAssign_c implements PolyJAssign {
    public PolyJFieldAssign_c(Position pos, Field left, Operator op, Expr right) {
        super(pos, left, op, right);
    }

    public void translate(CodeWriter w, Translator tr) {
        if (op == ASSIGN) {
            PolyJExt leftDel = (PolyJExt) left.ext();
            //HACK: this is *not* the right way to do this! This uses
            // a destructive update, rather than functional. It really
            // should be done with a visitor pass that replaces the
            // left node.
            this.left = (Expr) left.ext(leftDel.setLValue());
        }

        translateField(w, tr, (Field) left);
    }

    protected void translateField(CodeWriter w, Translator tr, Field left) {
        FieldInstance fi = left.fieldInstance();
        Context c = tr.context();
        PolyJTypeSystem ts = (PolyJTypeSystem) c.typeSystem();
        // Assuming here that we couldn't assign to a field of an Array.
        ClassType container = fi.container().toClass();
        ClassType uninst_container = ts.uninstantiate(container);
        FieldInstance origFi = ts.originalField(uninst_container, fi);
        Type declType = origFi.type();
        Type knownType = fi.type();
        // Does the field's container need a where object?
        boolean containerNeedsWhereObj = ts.needsWhereObject(container);
        // Is the field declared as a parameter?
        boolean declAsParam = declType instanceof ParamType;
        // Is the field a parametric type instantiated on a primitive?
        boolean instAsPrim = knownType.isPrimitive() && declAsParam;
        // Is the field instantiated on something other than a type parameter?
        boolean instTypeKnown = !(knownType instanceof ParamType);

        // If this is a static parametric field, and it
        // has not been instantiated on a primitive type, the
        // translation changes. Otherwise, do the normal thing.
        if (fi.flags().isStatic() && containerNeedsWhereObj && !instAsPrim) {
            PolyJTranslator pjtr = (PolyJTranslator) tr;

            // If we know the instantiated type, use the adapter class for
            // it to get the where object. Otherwise, the type must
            // be one of the current class's parameters, so use the
            // where object in the current class.
            if (ts.isFullyInst(container)) {
                translateStaticSet(w, tr, origFi, uninst_container, ts
                        .InstantiationBinding(container)
                        .getAdapterClassFullName()
                        + ".getInstance().");
            } else {
                translateStaticSet(w, tr, fi, uninst_container, "(("
                        + pjtr.adapterInterfaceFullName(c.currentClass()) + ")"
                        + pjtr.whereObjExpr() + ").");
            }
        } else {
            translateNormal(w, tr);
        }
    }

    protected void translateStaticSet(CodeWriter w, Translator tr,
            FieldInstance fi, ClassType enclosingPolyType,
            String adapterClassName) {

        Context c = tr.context();
        PolyJTypeSystem ts = (PolyJTypeSystem) c.typeSystem();
        TranslationTable tt = ts.translationTable();

        // Use the adapter class's $SetS$k method to set the
        // value of the static field.
        // If the value of this expression is used somewhere,
        // we need to do a cast.
        boolean isValueIgnored = ((PolyJExt) this.ext()).isValueIgnored();

        w.begin(0);
        if (!isValueIgnored) {
            w.write("((");
            if (left.type() instanceof InstType) {
                w.write(((InstType) left.type()).instType().toClass()
                        .translate(c));
            } else {
                w.write(left.type().translate(c));
            }
            w.write(")");
        }

        StaticFieldBinding fieldBinding = ts.StaticFieldBinding(
                enclosingPolyType, fi);

        w.write(adapterClassName + fieldBinding.set() + "(");
        w.allowBreak(2, " ");
        this.printSubExpr(right, w, tr);

        Package pack = enclosingPolyType.package_();
        String packageName = "";
        if (pack != null) {
            packageName = pack.fullName();
        }
        if (!fi.flags().isPublic()) {
            if (packageName == "") {
                w.write(", $token.the_instance");
            } else {
                w.write(", " + packageName + ".$token.the_instance");
            }
        }

        w.write(" )");
        if (!isValueIgnored) {
            w.write(")"); //close the cast
        }
        w.end();
    }

    protected void translateNormal(CodeWriter w, Translator tr) {
        super.translate(w, tr);
    }
}