package polyglot.ext.polyj.types;

import polyglot.types.*;
import polyglot.util.*;

import polyglot.ext.polyj.ast.*;
import java.util.*;

/**
 * A PolyJClassType represents classes that might also be polymorphic, i.e. classes
 * that could be instantiated.  Currently, all class are represented using this.
 */
public interface PolyJClassType extends ParsedClassType { 
    /**
     * Returns a List of ParamType
     */
    List params();

    /**
     * Returns the type representation of the parameter 
     * with the given name.
     */
    ParamType paramNamed(String name);

    /**
     * Returns true if this is a polymorphic type 
     * that can be instantiated.
     */
    boolean isPolymorphic();

    /**
     * Add a type parameter named paramName.
     */
    void addParam(String paramName);
    
    /**
     * Add the constraint to the type.
     */
    void addConstraint(ClassConstraint c) throws SemanticException;

    /**
     * Wrap the type in a lazy transformer.
     * See PolyJTypeWrapper.
     */
    WrappedClassType wrap(ClassType oldEnclosingType, ClassType newEnclosingType, PolyJTypeWrapper tw);
}
