package polyglot.ext.polyj.types;

import polyglot.types.*;
import polyglot.ext.polyj.visit.*;
import polyglot.util.*;

import polyglot.ext.jl.types.*;

import polyglot.ext.polyj.ast.*;

import java.util.*;

/** */
public class PolyJSubstType_c extends PolyJClassType_c implements WrappedClassType, InstType {
    protected PolyJClassType wrapped;
    protected final PolyJClassType r;
    protected final ClassType oldEnclosingType;
    protected /* final */ ClassType newEnclosingType;
    protected final PolyJTypeWrapper tw;
    protected Map methodMap;
    protected Map constructorMap;
    protected transient PolyJTypeSystem ts;

    PolyJSubstType_c(PolyJClassType r, 
                                  ClassType oldEnclosingType, 
                                  ClassType newEnclosingType, 
                                  PolyJTypeWrapper tw) {
        super(r.typeSystem(), r.typeSystem().defaultClassInitializer(), r.fromSource());

	
	while((r instanceof WrappedClassType)) {
	    WrappedClassType w = (WrappedClassType)r;
	    r = (PolyJClassType)w.wrappedClass();
	}
        this.wrapped = r;
        PolyJTypeSystem ts = (PolyJTypeSystem) r.typeSystem();
	


	this.oldEnclosingType = oldEnclosingType;
        this.newEnclosingType = newEnclosingType;        
	this.r = r; 
        this.tw = tw;
        this.methodMap = new HashMap();
        this.constructorMap = new HashMap();
        this.ts = (PolyJTypeSystem) r.typeSystem();
    }

    PolyJSubstType_c(Position pos, PolyJClassType instantiatedFrom,
                    List actualArgs, PolyJTypeWrapper tw)
    {
        this(instantiatedFrom, instantiatedFrom, null, tw);
	newEnclosingType = this;

    }

    PolyJSubstType_c(Position pos, PolyJClassType instantiatedFrom, 
                    List actualArgs, PolyJTypeSystem ts)
    {
        this(pos, instantiatedFrom, actualArgs, 
            ts.Wrapper(instantiatedFrom, actualArgs));
    }

    // Copied from InstType. Is oldEnclosingType correct here?
    // Does this still need to be overloaded in this combined version?
    /*
    public boolean inStaticContext() {
        return oldEnclosingType.inStaticContext();
    }
    */

    public List actualArgs() {
        return tw.args();
    }

    /**
     * true if the polymorphic type was instantiated on its own formal params
     */
    public boolean isNullInstantiation() {
        return actualArgs().equals(r.params());
    }

    public ClassType.Kind kind() {
        return r.kind();
    }
    
    public List methods() {
        return tw.methods(r, r, this, methodMap);
    }
    
    public List constructors() {
        return tw.constructors(r, r, this, constructorMap);
    }
    
    public MethodInstance originalMethod(MethodInstance mi) {
        return (MethodInstance) methodMap.get(mi);
    }
    
    public ConstructorInstance originalConstructor(ConstructorInstance ci) {
        return (ConstructorInstance) constructorMap.get(ci);
    }
    
    public List fields() {
        return tw.fields(r,r,this);
    }

    public Type superType() {
        return tw.superType(r, this);
    }

    public List interfaces() {
        return tw.interfaces(r, this);
    }

    public String fullName() { 
        return PolyJTranslator.fullNameForClass(r); 
    }
    
    public String name() { 
        return PolyJTranslator.nameForClass(r); 
    }
    
    public polyglot.types.Package package_() { 
        return r.package_(); 
    }
    
    public Flags flags() { 
        return r.flags(); 
    }
    
    public boolean isNested() { 
        return r.isNested(); 
    }
    
    public boolean isLocal() { 
        return r.isLocal(); 
    }
    public PolyJClassType instType() { 
	return r; 
    }
    
    public boolean isAnonymous() { 
        return r.isAnonymous(); 
    }
    
    // According to JavaDoc: outer class if this is an inner. 
    // Otherwise, null. This definitely doesn't look right.
    
    public ClassType outer() {
	//if (actualArgs().equals(r.params())) {
	    return r.outer();
	    //}
	    //return newEnclosingType;
    }
    
    
    /*
    // so... we'll play off the InstType version
    public ClassType outer() {
        return r.outer();
    }
    */
    // This ugly hack inherited in part from InstType_c
    public int hashCode() {


	if (actualArgs().equals(r.params())) {
	    return r.hashCode();
	}

	
	return ("" + r.hashCode() + (actualArgs().toString())).hashCode();
	
        // We can't include actualArgs in the hash because
        // if this is a null instantiatation, it
        // can be equal to a PolyJClassType.  A PJCT doesn't
        // have actuals, so including them here would
        // cause their hashes to differ.  That would break
        // the invariant that if two objects are equals()
        // they must have the same hashCode().
    }

    public boolean equalsImpl(TypeObject t) {
        PolyJTypeSystem ts = (PolyJTypeSystem) typeSystem();
        if (t instanceof ClassType) {
            ClassType c = (ClassType) t;
            if (ts.isInstantiated(c)) {
                return ts.isSameInst(this, c);
            }
        }
        return super.equalsImpl(t);
    }

    public boolean isEnclosedImpl(ClassType maybe_outer) {
        if (! isInner()) {
            return false;
        }

        PolyJTypeSystem ts = (PolyJTypeSystem) this.ts;

        if (! ts.isPolymorphicClass(maybe_outer)) {
            return false;
        }

        if (! r.isEnclosed(ts.uninstantiate(maybe_outer))) {
            return false;
        }

        return ts.equals(maybe_outer, tw.wrap(ts.uninstantiate(maybe_outer),
                                          r, this));
    }

    
    public List memberClasses() {
        return tw.memberClasses(r,r,this);
    }

    public ClassType memberClassNamed(String name) {
        return tw.memberClassNamed(name, r, r, this);
    }

    public String translate(Resolver c) {
        return r.translate(c);
    }

    ///////////////////////////////////////////////////////////

    //
    // Implement PolyDlg
    //
    
    public List params() {
        return r.params();
    }

    public ParamType paramNamed(String name) {
        return r.paramNamed(name);
    }

    public boolean isPolymorphic() {
        return r.isPolymorphic();
    }

    public void addParam(String paramName) {
        r.addParam(paramName);
    }

    public void addConstraint(ClassConstraint c) throws SemanticException{
        r.addConstraint(c);
    }

    public ClassType wrappedClass() {
        return r;
    }

    public List args() {
	    //hackhack.
        if (ts.equals(tw.wrappedType(),this)) {
            return tw.args();
        } else {
            return Collections.EMPTY_LIST;
        }
    }

    public PolyJTypeWrapper substitutor() {
        return tw;
    }

    
    public String toString() {
	if(newEnclosingType == this) {
	    if (ts.equals(this, instType())) {
		return instType().toString();
	    } else {
		String s = PolyJTranslator.instTypeString(instType(), actualArgs()); 
		return s;
	    }
	}

        if (newEnclosingType != null && ts.equals(newEnclosingType,wrappedClass())) {
            return newEnclosingType.toString();
        } else if (ts.equals(this, instType())) {
            return instType().toString();
        } else if (isTopLevel()) {
	    String s = PolyJTranslator.instTypeString(wrappedClass(), actualArgs()); 
	    return s;
        } else if (isMember() || isLocal()) {
	    String s = newEnclosingType + "." + name();
	    return s;
        } else {
            return "<anonymous member of " + outer + ">";
            // throw new InternalCompilerError("Unimplemented.");
        }
    }
    
}
